/******************************************************************************
 *   "$Id:  $"
 *
 *   DJ O'ksi'D Sampler.
 *
 *		   Copyright (c) David Monro 1996 (.wav struct)
 *                 Copyright (c) 2001  O'ksi'D
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *   Author : Jean-Marc Lienher ( http://oksid.ch )
 *
 ******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/soundcard.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <fcntl.h>
#include <curses.h>

typedef struct wavhead {
  u_long        main_chunk;     /* 'RIFF' */
  u_long        length;         /* Length of rest of file */
  u_long        chunk_type;     /* 'WAVE' */

  u_long        sub_chunk;      /* 'fmt ' */
  u_long        sc_len;         /* length of sub_chunk, =16 (rest of chunk) */
  u_short       format;         /* should be 1 for PCM-code */
  u_short       modus;          /* 1 Mono, 2 Stereo */
  u_long        sample_fq;      /* frequence of sample */
  u_long        byte_p_sec;
  u_short       byte_p_spl;     /* samplesize; 1 or 2 bytes */
  u_short       bit_p_spl;      /* 8, 12 or 16 bit */

  u_long        data_chunk;     /* 'data' */
  u_long        data_length;    /* samplecount (lenth of rest of block?)*/
} wavhead;


int key_switch(int c) {
	switch ((char)c) {
	case '': return 0;
	case '1': return 1;
	case '2': return 2;
	case '3': return 3;
	case '4': return 4;
	case '5': return 5;
	case '6': return 6;
	case '7': return 7;
	case '8': return 8;
	case '9': return 9;
	case '0': return 10;
	case 'q': return 11;
	case 'w': return 12;
	case 'e': return 13;
	case 'r': return 14;
	case 't': return 15;
	case 'z': return 16;
	case 'u': return 17;
	case 'i': return 18;
	case 'o': return 19;
	case 'p': return 20;
	case 'a': return 21;
	case 's': return 22;
	case 'd': return 23;
	case 'f': return 24;
	case 'g': return 25;
	case 'h': return 26;
	case 'j': return 27;
	case 'k': return 28;
	case 'l': return 29;
	case 'y': return 30;
	case 'x': return 31;
	case 'c': return 32;
	case 'v': return 33;
	case 'b': return 34;
	case 'n': return 35;
	case 'm': return 36;
	case ',': return 37;
	case '.': return 38;
	case '-': return 39;
	case '+':
		return -2;
	default:
		break;
	}
	return -1;
}

int load_wave(char *n, char **b) {
	int len = 0;
	char *p = n;
	char *buf;
	int thefd;
	wavhead wavhd;
	int count;
	long bcount = 0, bjump = 0;


	*b = NULL;
	while (*p != '\n' && *p != ' ' &&  *p != '\r' && *p != '\0') p++;
	*p = '\0';
	
	thefd = open(n, O_RDONLY, 0);
	if (!thefd) return 0;
	count = read(thefd, ((char*)&wavhd), sizeof(wavhd));
	if(wavhd.format != 1) return 0;

	len = wavhd.data_length * 2 * 2;
	buf = (char*) malloc(len);
	read(thefd, buf, len);

	*b = buf;
	return len;
}

int main(int argc, char **argv) 
{
	int current_sound = -1;
	int current_offset = 0;
	int samplesize = 16;
	int stereo = 1;
	int speed = 44100;
	int blksize = 1024;
	int fragment = 0x00030008;
	int afd;
	char *snd[80];
	int snd_len[80];
	int i;
	FILE *lfd;
	char buf[1024];
	

	for (i = 0; i < 80; i++) {
		snd[i] = NULL;
		snd_len[i] = 0;
	}
	
	if (argc < 2) {
		fprintf(stderr, "which file to use ?\n");
		exit(-1);
	}
	lfd = fopen(argv[1], "r");
	if (!lfd) {
		fprintf(stderr, "Error: cannot open sample file.");
		exit(-1);
	}
	i = 0;
	while (fgets(buf, 1024, lfd)) {
		snd_len[i] = load_wave(buf, &(snd[i]));
		i++;	
	}
	
	afd = open("/dev/dsp", O_WRONLY, 0);
	if (afd < 0) {
		fprintf(stderr, "Error: cannot open sound device");
		exit(-1);
	}
	ioctl(afd, SNDCTL_DSP_SAMPLESIZE, &samplesize);
	ioctl(afd, SNDCTL_DSP_STEREO, &stereo);
	ioctl(afd, SNDCTL_DSP_SPEED, &speed);
	//ioctl(afd, SNDCTL_DSP_GETBLKSIZE, &blksize);
	ioctl(afd, SNDCTL_DSP_SETFRAGMENT, &fragment);
//	ioctl(afd, SNDCTL_DSP_NONBLOCK);
//	ioctl(afd, SNDCTL_DSP_POST);

	initscr();
	//cbreak();
	nodelay(stdscr, TRUE);
	noecho();
	//timeout(0);
	//nodelay(stdscr, FALSE);
	
	while (current_sound > -2) {
		int c;
		c = getch();
		if (c != ERR) {
			current_sound = key_switch(c);
			current_offset = 0;
		} else if (current_sound > -1 && 
			snd_len[current_sound] > current_offset) 
		{
			int len;
			len = snd_len[current_sound] - current_offset;
			if (len > blksize) len = blksize;
			if (len > 128) len = 128;
			len = write(afd, snd[current_sound] + 
				current_offset, len);
			if (len > -1) {
				current_offset += len;
			} 
		} else {
			usleep(1000 * 10);
		}
	}

	endwin();
	close(afd);
}


/*
 * End of "$Id:  $".
 */
